import argparse

from .commands import build, test


def run(argv=None):
    parser = argparse.ArgumentParser(description="Top-level literary executable")
    subparsers = parser.add_subparsers(dest="command", required=True)

    for command in build, test:
        sub_parser = command.configure(subparsers)
        sub_parser.set_defaults(run=command.run)

    args = parser.parse_args(argv)
    args.run(args)


if __name__ == "__main__":
    run()



"""Notebook magic module

This module provides a namespace that can be imported into a Jupyter notebook
using `%run -m literary.notebook`.

It is implemented separately to `literary.notebook.__init__` in order to avoid mutating
an imported module (this module is regenerated for every execution)
"""
from . import *

update_namespace(globals())



import logging
import pathlib

import nbclient
import nbformat

logger = logging.getLogger(__name__)


def run_notebook(path: pathlib.Path, cell_timeout: int = 20):
    """Run a Jupyter notebook at the given path

    :param path: path to notebook
    :param cell_timeout: timeout for each cell execution
    :return:
    """
    try:
        logger.info(f"Executing {path}")
        nb = nbformat.read(path, as_version=nbformat.NO_CONVERT)
        client = nbclient.NotebookClient(
            nb, timeout=cell_timeout, resources={"metadata": {"path": path.parent}}
        )
        client.execute()
        logger.info(f"Finished executing {path}")
    except Exception:
        logger.exception(f"Error during execution of {path}")
        raise



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['literary', 'literary.commands', 'literary.notebook']

package_data = \
{'': ['*']}

install_requires = \
['Jinja2>=2.11.2,<3.0.0',
 'ipython>=7.19.0,<8.0.0',
 'nbclient>=0.5.1,<0.6.0',
 'nbconvert>=6.0.7,<7.0.0',
 'nbformat[fast]>=5.0.8,<6.0.0',
 'toml>=0.10.2,<0.11.0',
 'traitlets>=5.0.5,<6.0.0']

extras_require = \
{':python_version < "3.9"': ['astunparse>=1.6.3,<2.0.0']}

entry_points = \
{'console_scripts': ['literary = literary.__main__:run']}

setup_kwargs = {
    'name': 'literary',
    'version': '1.4.1',
    'description': 'Literate package development with Jupyter',
    'long_description': '![Literary logo with an orange cursive uppercase L inside black square brackets](assets/logo.png)\n\n# Literary \n[![pypi-badge][]][pypi] [![binder-badge][]][binder]  \n\n[binder]:\n  https://mybinder.org/v2/gh/agoose77/literary/HEAD?urlpath=lab%2Ftree%2Fexamples\n[binder-badge]: https://mybinder.org/badge_logo.svg\n[pypi-badge]: https://img.shields.io/pypi/v/literary\n[pypi]: https://pypi.org/project/literary\n\nThis package is an exploration of the [literate programming](http://www.literateprogramming.com) idea [pioneered by\n Donald\nKnuth](https://www-cs-faculty.stanford.edu/~knuth/lp.html) and implemented in the\n [`nbdev` package](https://github.com/fastai/nbdev). Although `nbdev` looks to be a very\nmature and comprehensive tool, it is quite opinionated. This package is an\ninvestigation into what a smaller `nbdev` might look like.\n\n## Philosophy\n1. **Low mental overhead**   \n Realistically, most Python programmers that wish to write packages need to have some\n familiarity with the Python package development model, including the conventional\nstructure of a package. For this reason, I feel that it is important to design\n`literary` such that these skills translate directly to designing libraries with\nnotebooks\n2. **Minimal downstream impact**  \n Users of `literary` packages should not realise that they are consuming \n notebook-generated code at runtime. This means that a pure-Python package needs to\n be generated from the notebooks, and it must use the conventional import model. For\n this reason, `literary` should only exist as a development dependency of\n the package.\n  \n\n## Differences with `nbdev`\n* Use of cell tags instead of comments or magics to dictate exports\n* Use of `nbconvert` machinery to build the pure-Python lib package\n* Use of import hooks to import other notebooks\n    * Maintains a similar programming model to conventional module\n development\n    * Reduces the need to modify notebook contents during conversion \n* Minimal runtime overhead\n    * Features like `patch` are removed from the generated module (& imported notebook source) using AST transformations\n* Currently no documentation generation\n    * Loosely, the plan is to use existing notebook-book tooling to re-use the\n     existing Jupyter ecosystem\n\n\n## Differences with Knuth\nKnuth introduced the `tangle` and `weave` programs to produce separate documentation and source code for compilation. \nLiterary differs in treating the notebook as the "ground truth" for documentation + testing, and generating smaller source code for packaging.\n\n\n## Design\nThe plan for this package is:\n1. Notebooks will be written inside `<PACKAGE_NAME>/` in literary project\'s root directory\n2. Notebooks will respect relative imports and other pure-Python features to minimise the differences between the generated packages and the notebooks\n3. A pure-python generated `lib/<PACKAGE_NAME>/` directory will be built before Poetry builds the final project.   \n  E.g. \n    ```toml\n    [tool.poetry]\n    # ...\n    packages = [\n      { include = "<PACKAGE_NAME>", from = "lib" },\n    ]\n    ```\n',
    'author': 'Angus Hollands',
    'author_email': 'goosey15@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/agoose77/literary',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import importlib.machinery
import linecache
import os
import pathlib
import sys
from typing import AnyStr, Union

import nbformat
import traitlets.config

from ..exporter import LiteraryPythonExporter


class NotebookLoader(importlib.abc.SourceLoader):
    def __init__(self, path):
        self.path = path

    def get_filename(self, full_name):
        return str(self.path)

    def update_linecache(self, path: str, source: str):
        linecache.cache[path] = (
            len(source),
            None,
            source.splitlines(keepends=True),
            path,
        )

    def get_data(self, path: str):
        c = traitlets.config.Config()
        nb = nbformat.read(path, as_version=nbformat.NO_CONVERT)
        exporter = LiteraryPythonExporter(config=c)
        body, resources = exporter.from_notebook_node(nb)
        # Ensure that generated source is available for tracebacks
        self.update_linecache(path, body)
        return body.encode()


class NotebookFinder(importlib.abc.MetaPathFinder):
    def __init__(self, path):
        self._path = path

    def find_spec(self, fullname, path, target=None):
        if path is None:
            path = self._path
        head, _, module_name = fullname.rpartition(".")

        for entry in path:
            # If we can find {module_name}/__init__.ipynb, this is a package
            package_path = pathlib.Path(entry) / module_name / "__init__.ipynb"
            if package_path.exists():
                spec = importlib.machinery.ModuleSpec(
                    fullname,
                    NotebookLoader(package_path),
                    origin=str(package_path),
                    is_package=True,
                )
                spec.submodule_search_locations = [str(package_path.parent)]
                spec.has_location = True
                return spec

            # Otherwise look for the notebook itself
            module_path = pathlib.Path(entry) / f"{module_name}.ipynb"
            if module_path.exists():
                spec = importlib.machinery.ModuleSpec(
                    fullname, NotebookLoader(module_path), origin=str(module_path)
                )
                spec.has_location = True
                return spec


def determine_package_name(path: pathlib.Path, package_root_path: pathlib.Path) -> str:
    """Determine the corresponding importable name for a package directory given by
    a particular file path

    :param path: path to package
    :param package_root_path: root path containing notebook package directory
    :return:
    """
    relative_path = path.relative_to(package_root_path)
    return ".".join(relative_path.parts)


def install_hook(package_root_path: Union[AnyStr, os.PathLike]):
    """Install notebook import hook

    Don't allow the user to specify a custom search path, because we also need this to
    interoperate with the default Python module importers which use sys.path

    :param package_root_path: root path containing notebook package directory
    :return:
    """
    # Make notebook packages importable by adding package root path to sys.path
    sys.path.append(str(package_root_path))
    sys.meta_path.insert(0, NotebookFinder(sys.path))
